﻿#Script By Olivier Gosselin - France
#http://www.purplescreen.eu
#Tools PowerCLI 6.5 Release 1

$CurentPath = [System.IO.Path]::GetDirectoryName($MyInvocation.MyCommand.Definition)
$Date = date | select *
$Day = $Date.Day
$Month = $Date.Month
$Hour = $Date.Hour
$Minute = $Date.Minute

#Recuperation de Variable pour d'autres functions
$Device = $script:Device
$VMName = $script:VMName
$GatewayIP = $script:Gateway
$guestosis = $script:guestosis
$CredentialWindows = $script:CredentialWindows
$CredentialLinux = $script:CredentialLinux

############### Function principale ###############
function Show-VM {
<#
.SYNOPSIS
This cmdlet retrieves useful informations of the virtual machines on a vCenter Server system.
.DESCRIPTION
.EXAMPLE
Show-VM -Name *
Show All VM
.EXAMPLE
Show-VM -Name MyVM1,MYVM2
Show VM MyVM1 and MYVM2
.EXAMPLE
Show-VM -Name * -SearchRoute 10.10.10.0
Search specifie route on All VM. You can see all result in FindRoute.log, in the same folder where you had run this script.
.INPUTS
None
.PARAMETER Name
Specifie the vm name.
.PARAMETER SearchRoute
Specifie IP Route what would you like to search of the virtual machines.
.LINK
http://www.purplescreen.eu
#>
[CmdletBinding()]
Param(
    [Parameter(Mandatory=$True,Position=0)]
        [array]$Name,

    [Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,Position=1)]
    [ValidateScript({$_ -match [IPAddress]$_ })]
        [string]$SearchRoute
)#PARAM

#On vide les variables suivantes
$script:Device = @()
$script:VMName = @()
$script:Gateway = @()

$VMGuestVM = get-vm -Name $Name
$VMGuest = $VMGuestVM | Get-VMGuest

   foreach ($VMs in $VMGuest) {
$Hostname = $VMs.HostName
$VMGuestIPs = $VMs.nics
$ToolsStatus = $VMs.ExtensionData.ToolsStatus

#On verifie l'etat des VMwaretools
if (($ToolsStatus -eq "toolsOk") -or ($ToolsStatus -eq "toolsOld")) {
if ($ToolsStatus -eq "toolsOld") {
Write-Host "Les VMwareTools ne sont pas a jour pour cette VM $VMs" -ForegroundColor Yellow
erreur
}
    foreach ($NetIP in $VMGuestIPs) {
#Pour l'affichage et mise en variable Route
$VMName = $VMs.VmName

$NetworkRoute = $VMs.ExtensionData.IpStack.IpRouteConfig.IpRoute.Network
$PrefixLength = $VMs.ExtensionData.IpStack.IpRouteConfig.IpRoute.PrefixLength
$IpAddressRoute = $VMs.ExtensionData.IpStack.IpRouteConfig.IpRoute.Gateway.IpAddress

$StateCard = $NetIP.Connected
$Device = $NetIP.Device
$IP = $NetIP.IPAddress | select -Unique
$MAC = $NetIP.MacAddress

$WhereisMAC = $VMGuestIPs.MacAddress
$WhereisGateway = [array]::indexof($WhereisMAC,"$MAC")
$Gateway = $VMs.ExtensionData.IpStack.IpRouteConfig.IpRoute.Gateway | where {$_.Device -eq "$WhereisGateway"} | where {$_.IpAddress -ne $null}
$GatewayIP = $Gateway.IpAddress | select -Unique

if ($SearchRoute) {
$ForArrayMask = 0
    foreach ($Route in $NetworkRoute) {
$NetworkRoute2 = $Route
$PrefixLength2 = $PrefixLength[$ForArrayMask]
$IpAddressRoute2 = $IpAddressRoute[$ForArrayMask]

$ForArrayMask++
$CheckSubnetIPSearch = checkSubnet -cidr "$NetworkRoute2/$PrefixLength2" -ip "$SearchRoute"
    if ($CheckSubnetIPSearch -eq $True) {
$NetworkRoute = $NetworkRoute2
if (!$PrefixLength2) {
$PrefixLength2 = $null
}
else {
$PrefixLength = $PrefixLength2
}
if ($IpAddressRoute2 -eq $null) {
$IpAddressRoute = "On-link" 
}
else {
$IpAddressRoute = $IpAddressRoute2
}
$CheckSubnetIPOK = $True
}
}
}

################# AFFICHAGE ###################
    if (!$IP) {
$IP = "None"
}
    else {
$IP = (@($IP) -join ',')
}

    if (!$Device -or !$MAC -or !$GatewayIP -or !$NetworkRoute -or !$PrefixLength -or !$IpAddressRoute) {
$GatewayIP = "None"
$NetworkRoute = "None"
$PrefixLength = "None"
$IpAddressRoute = "None"
$MAC = "None"
$Device = "Unknown"
}

#Passage de Variable Mettre ces variables vide en debut de function
$script:Device += @($Device)
$script:VMName += @($VMName)
$script:gateway += @($GatewayIP)

#Pour l'affichage et mise en variable Gateway & Cie
$NameOutput = New-Object psobject
$NameOutput | Add-Member -MemberType NoteProperty -Name VM -Value $VMName
$NameOutput | Add-Member -MemberType NoteProperty -Name Hostname -Value $Hostname
$NameOutput | Add-Member -MemberType NoteProperty -Name StateCard -Value $StateCard
$NameOutput | Add-Member -MemberType NoteProperty -Name Device -Value $Device
$NameOutput | Add-Member -MemberType NoteProperty -Name IP -Value $IP
$NameOutput | Add-Member -MemberType NoteProperty -Name MAC -Value $MAC
$NameOutput | Add-Member -MemberType NoteProperty -Name Gateway -Value $GatewayIP
$NameOutput | Add-Member -MemberType NoteProperty -Name StateVMwareTools -Value $ToolsStatus

#Pour l'affiche des routes
$NetworkRoute = (@($NetworkRoute) -join '|')
$PrefixLength = (@($PrefixLength) -join '|')
$IpAddressRoute = (@($IpAddressRoute) -join '|')
$RouteOutput = New-Object psobject
$RouteOutput | Add-Member -MemberType NoteProperty -Name VM -Value $VMName
$RouteOutput | Add-Member -MemberType NoteProperty -Name NetworkRoute -Value $NetworkRoute
$RouteOutput | Add-Member -MemberType NoteProperty -Name PrefixLength -Value $PrefixLength
$RouteOutput | Add-Member -MemberType NoteProperty -Name IpAddressRoute -Value $IpAddressRoute
$RouteOutput | Add-Member -MemberType NoteProperty -Name StateVMwareTools -Value $ToolsStatus

#On affiche le resultat
if ($SearchRoute -and $Name) {
if ($CheckSubnetIPOK -eq "True") {
$RouteOutput | where {$_.NetworkRoute -ne "None"}
### Creation du fichier log
Add-Content -Path $CurentPath"\FindRoute.log" -Value "[$Day/$Month-$Hour.$Minute] $VMName,$NetworkRoute/$PrefixLength"
$CheckSubnetIPOK = @()
}
}
elseif ($Name) {
$NameOutput
}
}
}
else {
### Creation du fichier log avec les erreurs
erreur
}
}
}

function Set-DefaultGateway {
<#
.SYNOPSIS
This cmdlet set a gateway of the virtual machines on a vCenter Server system.
.DESCRIPTION
.EXAMPLE
Set-DefaultGateway -Name MyVM -GatewayIN 10.10.10.1 -GatewayOUT 10.10.10.254
Set your default Gateway for virtual machine name is "MyVM" from gateway 10.10.10.1 to Gateway 10.10.10.254
.INPUTS
None
.PARAMETER Name
Specifie the vm name.
.PARAMETER GatewayIN
Specifie IP Gateway to change in for the virtual machine(s).
.PARAMETER GatewayOUT
Specifie a new IP Gateway to default for the virtual machine(s).
.LINK
http://www.purplescreen.eu
#>
[CmdletBinding()]
Param(
    [Parameter(Mandatory=$True,Position=0)]
        [string]$Name,

    [Parameter(Mandatory=$True,ValueFromPipelineByPropertyName=$true,Position=1)]
    [ValidateScript({$_ -match [IPAddress]$_ })]
        [string]$GatewayIN,

    [Parameter(Mandatory=$True,ValueFromPipelineByPropertyName=$true,Position=2)]
    [ValidateScript({$_ -match [IPAddress]$_ })]
        [string]$GatewayOUT
)

$script:VMwareTools = @()

$AllVMName = Get-vm -Name $Name | where {$_.PowerState -eq "PoweredOn"}

foreach ($VM in $AllVMName) {
$OSis = getvmos -VMName $VM
$OSfindis = $script:OSis

foreach ($ThisOS in $guestosis) {
#Variable pour affiche du nom de la VM
$GetVMName = $VM

#Verifier les VMware Tools
CheckVMTools -Name $VM

if ($VMwareTools -eq "True") {
#Demande d'authentification de la VM
getcredvm -getvmos $ThisOS

#On verifie le type d'OS
if ($ThisOS -eq "windowsGuest") {
$CredentialWindows = $script:CredentialWindows

#Script pour les machines windows
$scriptWindows =  @'
$Forchange = Get-NetIPConfiguration | where {$_.IPv4DefaultGateway.NextHop -eq "#GatewayIN#"}
$GetPrefixLength = Get-NetIPAddress | where {$_.InterfaceAlias -eq "$InterfaceAlias"}
$DestinationPrefix = '0.0.0.0/0'
$InterfaceAlias = $Forchange.InterfaceAlias
$InterfaceIndex = $Forchange.InterfaceIndex
$InterfaceDescription = $Forchange.InterfaceDescription
$IPv4Address = $Forchange.IPv4Address
$IPv4DefaultGateway = $Forchange.IPv4DefaultGateway
$DNSServer = $Forchange.DNSServer
$GetNetIPAddress = Get-NetIPAddress | where {$_.InterfaceAlias -eq "$InterfaceAlias"}
$PrefixLength = $GetNetIPAddress.PrefixLength
$InterfaceIndex = $GetNetIPAddress.InterfaceIndex
$IPAddress = $GetNetIPAddress.IPAddress
Remove-NetRoute -InterfaceIndex $InterfaceIndex -NextHop "#GatewayIN#" -confirm:$false
New-NetRoute -InterfaceIndex $InterfaceIndex -NextHop #GatewayOUT# -DestinationPrefix $DestinationPrefix -confirm:$false
'@

#Example pour modifier le script envoyer au server windows
$scriptWindows = $scriptWindows.Replace('#GatewayIN#',$GatewayIN).Replace('#GatewayOUT#',$GatewayOUT)
Invoke-VMScript -vm $VM -guestUser $script:CredentialWindows.UserName -guestpassword $script:CredentialWindows.Password -scripttype powershell -scripttext $scriptWindows
}
else {
$CredentialLinux = $script:CredentialLinux
#tableau=($(route -n | grep #GatewayIN#))
#Script pour les machines machine linux
#ip route change ${tableau[0]} via #GatewayOUT# dev ${tableau[4]}
$scriptAutre =  @'
ip route show | grep #GatewayIN#
echo " "
echo "Non supporter sur Unix pour le moment."
'@
$scriptAutre = $scriptAutre.Replace('#GatewayIN#',$GatewayIN).Replace('#GatewayOUT#',$GatewayOUT)
Invoke-VMScript -vm $VM -guestUser $CredentialLinux.UserName -guestpassword $CredentialLinux.Password -scripttype bash -scripttext $scriptAutre
}
}
else {
Write-Host "Action annuler pour la VM $VM. Les VMWaretools ne fonctionne pas"
}
}
}
}

function Remove-Route {
<#
.SYNOPSIS
This cmdlet remove a route of the virtual machines on a vCenter Server system.
.DESCRIPTION
.EXAMPLE
Remove-Route -Name MyVM -Destination 192.168.4.0
Remove-Route 192.168.4.0 on VM MyVM
.INPUTS
None
.PARAMETER Name
Specifie the vm name.
.PARAMETER Destination
Specifie IP.......
.LINK
http://www.purplescreen.eu
#>
[CmdletBinding()]
Param(
    [Parameter(Mandatory=$True,Position=0)]
        [array]$Name,

    [Parameter(Mandatory=$True,Position=1)]
    [ValidateScript({$_ -match [IPAddress]$_ })]
        [string]$Destination
)

$script:VMwareTools = @()

$AllVMName = Get-vm -Name $Name | where {$_.PowerState -eq "PoweredOn"}

foreach ($VM in $AllVMName) {
$OSis = getvmos -VMName $VM
$OSfindis = $script:OSis

foreach ($ThisOS in $guestosis) {
#Variable pour affiche du nom de la VM
$GetVMName = $VM

#Verifier les VMware Tools
CheckVMTools -Name $VM

if ($VMwareTools -eq "True") {
#Demande d'authentification de la VM
getcredvm -getvmos $ThisOS

#On verifie le type d'OS
if ($ThisOS -eq "windowsGuest") {
$CredentialWindows = $script:CredentialWindows

#Script pour les machines windows
$scriptWindowsDel = @'
route delete #Destination#
'@
#Passage de variables et envoye du script vers la machine distante
$scriptWindowsDel = $scriptWindowsDel.Replace('#Destination#',$Destination)
Invoke-VMScript -vm $VM -guestUser $script:CredentialWindows.UserName -guestpassword $script:CredentialWindows.Password -scripttype powershell -scripttext $scriptWindowsDel
}
else {
$CredentialLinux = $script:CredentialLinux
$scriptAutreDel =  @'
var=$(ip route show | grep #Destination#)
ip route del ${var}
'@
#Passage de variables et envoye du script vers la machine distante
$scriptAutreDel = $scriptAutreDel.Replace('#Name#',$Name).Replace('#Destination#',$Destination)
Invoke-VMScript -vm $VM -guestUser $CredentialLinux.UserName -guestpassword $CredentialLinux.Password -scripttype bash -scripttext $scriptAutreDel
}
}
else {
Write-Host "Action annuler pour la VM $VM. Les VMWaretools ne fonctionne pas"
}
}
}
}

function Update-Route {
<#
.SYNOPSIS
This cmdlet change a gateway of the virtual machines on a vCenter Server system.
.DESCRIPTION
.EXAMPLE
Update-Route -Name MyVM -Destination 192.168.4.0 -Mask 255.255.254.0 -Gateway 10.10.10.1
Update-Route route 192.168.0.4 on VM MyVM with this new setting.
.INPUTS
None
.PARAMETER Name
Specifie the vm name.
.PARAMETER Destination
Specifie IP.......
.PARAMETER Mask
Specifie IP Mask .......
.PARAMETER Gateway
Specifie IP Gateway .......
.LINK
http://www.purplescreen.eu
#>
[CmdletBinding()]
Param(
    [Parameter(Mandatory=$True,Position=0)]
        [array]$Name,

    [Parameter(Mandatory=$True,Position=1)]
    [ValidateScript({$_ -match [IPAddress]$_ })]
        [string]$Destination,

    [Parameter(Mandatory=$True,Position=2)]
    [ValidateScript({$_ -match [IPAddress]$_ })]
        [string]$Mask,

    [Parameter(Mandatory=$True,Position=3)]
    [ValidateScript({$_ -match [IPAddress]$_ })]
        [string]$Gateway

)

$script:VMwareTools = @()

$AllVMName = Get-vm -Name $Name | where {$_.PowerState -eq "PoweredOn"}

foreach ($VM in $AllVMName) {
$OSis = getvmos -VMName $VM
$OSfindis = $script:OSis

foreach ($ThisOS in $guestosis) {
#Variable pour affiche du nom de la VM
$GetVMName = $VM

#Verifier les VMware Tools
CheckVMTools -Name $VM

if ($VMwareTools -eq "True") {
#Demande d'authentification de la VM
getcredvm -getvmos $ThisOS

#On verifie le type d'OS
if ($ThisOS -eq "windowsGuest") {
$CredentialWindows = $script:CredentialWindows

#route ip change #Destination# MASK #Mask# #Gateway#
$scriptWindowsCHANGE =  @'
route change #Destination# MASK #Mask# #Gateway#
'@
#Passage de variables et envoye du script vers la machine distante
$scriptWindowsCHANGE = $scriptWindowsCHANGE.Replace('#Destination#',$Destination).Replace('#Mask#',$Mask).Replace('#Gateway#',$Gateway)
Invoke-VMScript -vm $VM -guestUser $script:CredentialWindows.UserName -guestpassword $script:CredentialWindows.Password -scripttype powershell -scripttext $scriptWindowsCHANGE
}
else {
$CredentialLinux = $script:CredentialLinux
$scriptLinuxCHANGE =  @'
ip route show
echo " "
echo "Non supporter sur Unix pour le moment."
'@
#Passage de variables et envoye du script vers la machine distante
$scriptLinuxCHANGE = $scriptLinuxCHANGE.Replace('#Name#',$Name).Replace('#Destination#',$Destination).Replace('#Mask#',$Mask).Replace('#Gateway#',$Gateway)
Invoke-VMScript -vm $VM -guestUser $CredentialLinux.UserName -guestpassword $CredentialLinux.Password -scripttype bash -scripttext $scriptLinuxCHANGE
}
}
else {
Write-Host "Action annuler pour la VM $VM. Les VMWaretools ne fonctionne pas"
}
}
}
}

function Add-Route {
<#
.SYNOPSIS
This cmdlet add a gateway of the virtual machines on a vCenter Server system.
.DESCRIPTION
.EXAMPLE
Add-Route -Name MyVM -Destination 192.168.4.0 -Mask 255.255.254.0 -Gateway 10.10.10.1 -Persistant False
Add-Route on VM MyVM
.INPUTS
None
.PARAMETER Name
Specifie the vm name.
.PARAMETER Destination
Specifie IP.......
.PARAMETER Mask
Specifie IP Mask .......
.PARAMETER Gateway
Specifie IP Gateway .......
.PARAMETER Persistant
Specifie  .......
.LINK
http://www.purplescreen.eu
#>
[CmdletBinding()]
Param(
    [Parameter(Mandatory=$True,Position=0)]
        [array]$Name,

    [Parameter(Mandatory=$True,Position=1)]
    [ValidateScript({$_ -match [IPAddress]$_ })]
        [string]$Destination,

    [Parameter(Mandatory=$True,Position=2)]
    [ValidateScript({$_ -match [IPAddress]$_ })]
        [string]$Mask,

    [Parameter(Mandatory=$True,Position=3)]
    [ValidateScript({$_ -match [IPAddress]$_ })]
        [string]$Gateway,

    [Parameter(Mandatory=$False,Position=4)]
    [ValidateSet("False", "True")]
        [string[]]$Persistant
)
if (!$Persistant) {
$Persistant = "True"
}
$script:VMwareTools = @()

$AllVMName = Get-vm -Name $Name | where {$_.PowerState -eq "PoweredOn"}

foreach ($VM in $AllVMName) {
$OSis = getvmos -VMName $VM
$OSfindis = $script:OSis

foreach ($ThisOS in $guestosis) {
#Variable pour affiche du nom de la VM
$GetVMName = $VM

#Verifier les VMware Tools
CheckVMTools -Name $VM

if ($VMwareTools -eq "True") {
#Demande d'authentification de la VM
getcredvm -getvmos $ThisOS

#On verifie le type d'OS
if ($ThisOS -eq "windowsGuest") {
$CredentialWindows = $script:CredentialWindows

$scriptWindowsADD =  @'
function Convert-IpAddressToMaskLength([string] $dottedIpAddressString) {
  $result = 0; 
  # ensure we have a valid IP address
  [IPAddress] $ip = $dottedIpAddressString;
  $octets = $ip.IPAddressToString.Split('.');
  foreach($octet in $octets)
  {
    while(0 -ne $octet) 
    {
      $octet = ($octet -shl 1) -band [byte]::MaxValue
      $result++; 
    }
  }
  return $result;
}

function checkSubnet {
Param(
    [Parameter(Mandatory=$True,Position=1)]
        [string]$cidr,

    [Parameter(Mandatory=$True,Position=1)]
        [string]$ip

)
    $network, [int]$subnetlen = $cidr.Split('/')

#IF IPv6  
if ($network -match ":") {
#Code IPv6
}
#IF IPv4
else {
    $a = [uint32[]]$network.split('.')
    
    [uint32] $unetwork = ($a[0] -shl 24) + ($a[1] -shl 16) + ($a[2] -shl 8) + $a[3]
    $mask = (-bnot [uint32]0) -shl (32 - $subnetlen)
    $a = [uint32[]]$ip.split('.')
    
    [uint32] $uip = ($a[0] -shl 24) + ($a[1] -shl 16) + ($a[2] -shl 8) + $a[3]
    $unetwork -eq ($mask -band $uip)
    }
}

$Routeresult = @()

$SearchMASK = "#Mask#"
$SearchNextHop = "#Gateway#"
$SearchDestination = "#Destination#"
$Find = get-netroute
$FindNextHop = $Find.NextHop
$FindDestination = $Find.DestinationPrefix

$MaskIs = (Convert-IpAddressToMaskLength $SearchMASK)

foreach ($isDestination in $FindDestination) {
$Routeresult += @($RouteExist)

$SubnetExist = (checkSubnet -cidr "$isDestination" -ip "$SearchDestination")
    if ($SubnetExist -eq "True") {
$RouteExist = "True"
}
elseif ($SubnetExist -eq "False") {
$RouteExist = "False"
}
elseif ($SubnetExist -eq $null) {
$RouteExist = "False"
}
}

$CheckRouteExist = [array]::indexof($Routeresult,"True")
    if ($CheckRouteExist -ne "-1") {
Write-Host "La Route exist."
}
else {
Write-Host "La route est ajoute."
Write-Host ""
route add #Destination# MASK #Mask# #Gateway# #Persistant#
}
'@
if ($Persistant -eq "True") {
$Persistant2 = "-p"
}
#Passage de variables et envoye du script vers la machine distante
$scriptWindowsADD = $scriptWindowsADD.Replace('#Destination#',$Destination).Replace('#Mask#',$Mask).Replace('#Gateway#',$Gateway).Replace('#Persistant#',$Persistant2)
Invoke-VMScript -vm $VM -guestUser $script:CredentialWindows.UserName -guestpassword $script:CredentialWindows.Password -scripttype powershell -scripttext $scriptWindowsADD
}
else {
$CredentialLinux = $script:CredentialLinux
$scriptAutreADD =  @'
ip route add #Destination#/#Mask# via #Gateway#
'@

#ip route add #Destination#/#Mask# via #Gateway#
$MaskIs = (Convert-IpAddressToMaskLength $Mask)

#Passage de variables et envoye du script vers la machine distante
$scriptAutreADD = $scriptAutreADD.Replace('#Name#',$Name).Replace('#Destination#',$Destination).Replace('#Mask#',$MaskIs).Replace('#Gateway#',$Gateway)
Invoke-VMScript -vm $VM -guestUser $CredentialLinux.UserName -guestpassword $CredentialLinux.Password -scripttype bash -scripttext $scriptAutreADD
}
}
else {
Write-Host "Action annuler pour la VM $VM. Les VMWaretools ne fonctionne pas"
}
}
}
}

############### Function Complementaire ###############
#fonction Log en cas d'erreur
function erreur {
$VMName = $VMs.VmName
$PowerVM = get-vm -Name $VMName
$ToolsStatus = $PowerVM.ExtensionData.Guest.ToolsStatus
$ToolsRunning = $PowerVM.ExtensionData.Guest.ToolsRunningStatus
$ToolsVersion = $PowerVM.ExtensionData.Guest.ToolsVersionStatus

    if ($PowerVM.PowerState -eq "PoweredOff") {
$ErreurVM = "Erreur : La VM ($VMName) est eteinte"
}

    elseif ($ToolsStatus -eq "toolsNotRunning") {
$ErreurVM = "Erreur : Les VMWare Tool ($VMName) ne fonctionne pas"
}
    elseif ($ToolsStatus -eq "toolsOld") {
$ErreurVM = "Avertissement : Les VMWare tools pour cette VM ($VMName) ne sont pas a jour"
}
    elseif ($ToolsVersion -ne "guestToolsCurrent") {
$ErreurVM = "Erreur : Les VMWare tools pour cette VM ($VMName) ne sont pas installer"
}

#On ecrit les erreurs dans un fichier TXT
Add-Content -Path $CurentPath"\Erreur.log" -Value "[$Day/$Month-$Hour.$Minute]$ErreurVM"
$ToolsStatus = @()
}

function getvmos ($VMName) {
$script:guestosis = @()
$VM = get-vm -Name $VMName
$guestosis = $VM.ExtensionData.Guest.GuestFamily
$script:guestosis += @($guestosis)
}

function getcredvm ($getvmos) {
$script:CredentialWindows = @()
$script:CredentialLinux = @()
$getvmos = $script:guestosis
    if ($getvmos -eq "windowsGuest") {
$CredentialWindows = Get-Credential -Message "Identifiants pour la VM ($GetVMName)"
}
    elseif ($getvmos -eq "linuxGuest") {
$CredentialLinux = Get-Credential -Message "Identifiants pour la VM ($GetVMName)"
}
    elseif ($getvmos -eq "otherGuestFamily") {
$CredentialLinux = Get-Credential -Message "Identifiants pour la VM ($GetVMName)"
}
#$VM = get-vm -Name $VMName
#$guestosis = $VM.ExtensionData.Guest.GuestFamily
$script:CredentialWindows += @($CredentialWindows)
$script:CredentialLinux += @($CredentialLinux)
}

function checkSubnet {
Param(
    [Parameter(Mandatory=$True,Position=1)]
        [string]$cidr,

    [Parameter(Mandatory=$True,Position=1)]
        [string]$ip

)
    $network, [int]$subnetlen = $cidr.Split('/')

#IF IPv6  
if ($network -match ":") {
#Code IPv6
}
#IF IPv4
else {
    $a = [uint32[]]$network.split('.')
    
    [uint32] $unetwork = ($a[0] -shl 24) + ($a[1] -shl 16) + ($a[2] -shl 8) + $a[3]
    $mask = (-bnot [uint32]0) -shl (32 - $subnetlen)
    $a = [uint32[]]$ip.split('.')
    
    [uint32] $uip = ($a[0] -shl 24) + ($a[1] -shl 16) + ($a[2] -shl 8) + $a[3]
    $unetwork -eq ($mask -band $uip)
    }
}

function CheckVMTools ($Name) {
$script:VMwareTools = @()
$VMGuestVM = get-vm -Name $Name
$VMGuest = $VMGuestVM | Get-VMGuest
$Hostname = $VMGuest.HostName
$VMGuestIPs = $VMGuest.nics
$ToolsStatus = $VMGuest.ExtensionData.ToolsStatus

#On verifie l'etat des VMwaretools
if ($ToolsStatus -eq "toolsOk") {
$VMwareTools = "True"
}
else {
$VMwareTools = "False"
}
#Write-Host "VMTool is $VMwareTools"
$script:VMwareTools += @($VMwareTools)
}

function Convert-IpAddressToMaskLength([string] $dottedIpAddressString)
{
  $result = 0; 
  # ensure we have a valid IP address
  [IPAddress] $ip = $dottedIpAddressString;
  $octets = $ip.IPAddressToString.Split('.');
  foreach($octet in $octets)
  {
    while(0 -ne $octet) 
    {
      $octet = ($octet -shl 1) -band [byte]::MaxValue
      $result++; 
    }
  }
  return $result;
}